### Clear memory and set random seed ===========================================
rm(list = ls()) # clear memory
graphics.off()  # clear all plots
cat("\014")     # clear console (same as Ctrl-L in console)
packages = c("readr","readxl")
#use this function to check if each package is on the local machine
#if a package is installed, it will be loaded
#if any are not, the missing package(s) will be installed and loaded
package.check <- lapply(packages, FUN = function(x) {
  if (!require(x, character.only = TRUE)) {
    install.packages(x, dependencies = TRUE)
    library(x, character.only = TRUE)
  }
})

# set main directory
setwd('~/Box Sync/ASSR2/results')
#setwd('C:/Users/mszyc//Box Sync/ASSR2/results')
#setwd('~/Malina_Box/Box Sync/ASSR2/results')

dir_wd = getwd()

### READ IN ALL SUBJECTS RAW DATA AND ADD EXTRA VARIABLES ======================
dir_log = "~/Box Sync/ASSR2/logfiles"
#dir_log = "C:/Users/mszyc/Box Sync/ASSR2/logfiles"
#dir_log = "~/Malina_Box/Box Sync/ASSR2/logfiles"
setwd(dir_log)

files = dir(dir_log, pattern = 'fp')
df = data.frame()
fps = numeric()
for (f in 1:length(files)){
  # f = 1
  tmp = read.csv(files[f], sep = '\t', header = T, stringsAsFactors = F)
  fps = c(fps, tmp$fp[1])
  tmp$rel_trials = 0
  tmp$rel_trials[grep(pattern = 1, x = tmp$target)] = 1
  tmp$rel_trials[grep(pattern = 1, x = tmp$target)+1] = 2
  tmp$hit = 0
  tmp$RT = 0
  tmp$FA = 0
  tmp$key_press = 0
  
  # key_press: 
  # 1 for key press during target trial, 
  # 2 for key press on the trial following the target, 
  # 3 for key press with no target (False alarm)
  
  # check first for delayed response, then for a response during the actual trial. 
  # If there was an earlier response (during the actual trial), use that one (because it was first)
  for (trial in 1:length(tmp$response_time[tmp$rel_trials == 2])) {
    if (tmp$response_time[tmp$rel_trials == 2][trial] != 'NaN') {
      tmp$RT[tmp$rel_trials == 1][trial] = tmp$response_time[tmp$rel_trials==2][trial] - tmp$trial_start[tmp$rel_trials==1][trial]
      tmp$key_press[tmp$rel_trials == 1][trial] = 2
    }
  }

  for (trial in 1:length(tmp$response_time[tmp$rel_trials == 1])) {
    if (tmp$response_time[tmp$rel_trials == 1][trial] != 'NaN') {
      tmp$RT[tmp$rel_trials == 1][trial] = tmp$response_time[tmp$rel_trials==1][trial] - tmp$trial_start[tmp$rel_trials==1][trial]
      tmp$key_press[tmp$rel_trials == 1][trial] = 1
    }
  }

  # the above code for key_press 2 would be incorrect if the last trial in a block is a target trial.
  if (any(tmp$trial_nr == 247 & tmp$rel_trials == 1)) {
    print(paste0('Subject = ', fps))
    print('Check data... ')
    print(which(tmp$trial_nr == 247 & tmp$rel_trials == 1))
    stop('Software error: keypress 2 is assigned for subsequent block')
  }
  
  for (trial in 1:length(tmp$response_time[tmp$rel_trials == 0])) {
    if (tmp$response_time[tmp$rel_trials == 0][trial] != 'NaN') {
      tmp$key_press[tmp$rel_trials == 0][trial] = 3
      tmp$FA[tmp$rel_trials == 0][trial] = 1
    }
  }
  
  tmp$hit[tmp$RT >= 0.2 & tmp$RT <= 1] = 1 
  #tmp$hit[tmp$RT >= 0.2 & tmp$RT <= 0.7] = 1 # if you want a stricter hit criterion
  #tmp$FA[tmp$RT < 0.2 | tmp$RT > 0.7] = 1
  
  df=rbind(df,tmp)
  
}
rm(f,tmp)

fps = sort(fps)

df$tone[df$tone=="20Hz_AM_inv"] = "20Hz_AM"
df$tone[df$tone=="40Hz_AM_inv"] = "40Hz_AM"
df$tone[df$tone=="80Hz_AM_inv"] = "80Hz_AM"


# Save performance data
outTable <- data.frame()
for (indx in 1:length(fps)){
  fp = fps[indx]
  outTable[indx, 1]  <- fp
  outTable[indx, 2]  <- sum(df$hit[df$fp == fp & df$load == "low"  & df$tone == "20Hz_AM"]) 
  outTable[indx, 3]  <- sum(df$hit[df$fp == fp & df$load == "low"  & df$tone == "40Hz_AM"]) 
  outTable[indx, 4]  <- sum(df$hit[df$fp == fp & df$load == "low"  & df$tone == "80Hz_AM"]) 
  outTable[indx, 5]  <- sum(df$hit[df$fp == fp & df$load == "high" & df$tone == "20Hz_AM"]) 
  outTable[indx, 6]  <- sum(df$hit[df$fp == fp & df$load == "high" & df$tone == "40Hz_AM"]) 
  outTable[indx, 7]  <- sum(df$hit[df$fp == fp & df$load == "high" & df$tone == "80Hz_AM"]) 
  outTable[indx, 8]  <- sum(df$FA[df$fp == fp  & df$load == "low"  & df$tone == "20Hz_AM"]) 
  outTable[indx, 9]  <- sum(df$FA[df$fp == fp  & df$load == "low"  & df$tone == "40Hz_AM"]) 
  outTable[indx, 10] <- sum(df$FA[df$fp == fp  & df$load == "low"  & df$tone == "80Hz_AM"]) 
  outTable[indx, 11] <- sum(df$FA[df$fp == fp  & df$load == "high" & df$tone == "20Hz_AM"]) 
  outTable[indx, 12] <- sum(df$FA[df$fp == fp  & df$load == "high" & df$tone == "40Hz_AM"]) 
  outTable[indx, 13] <- sum(df$FA[df$fp == fp  & df$load == "high" & df$tone == "80Hz_AM"]) 
  outTable[indx, 14] <- mean(df$RT[df$fp == fp & df$hit == 1 & df$load == "low" & df$tone == "20Hz_AM"]) * 1000
  outTable[indx, 15] <- mean(df$RT[df$fp == fp & df$hit == 1 & df$load == "low" & df$tone == "40Hz_AM"]) * 1000
  outTable[indx, 16] <- mean(df$RT[df$fp == fp & df$hit == 1 & df$load == "low" & df$tone == "80Hz_AM"]) * 1000
  outTable[indx, 17] <- mean(df$RT[df$fp == fp & df$hit == 1 & df$load == "high" & df$tone == "20Hz_AM"]) * 1000
  outTable[indx, 18] <- mean(df$RT[df$fp == fp & df$hit == 1 & df$load == "high" & df$tone == "40Hz_AM"]) * 1000
  outTable[indx, 19] <- mean(df$RT[df$fp == fp & df$hit == 1 & df$load == "high" & df$tone == "80Hz_AM"]) * 1000
}
colnames(outTable) <- c('fp', 
                        'nHRlo20Hz', 'nHRlo40Hz', 'nHRlo80Hz',
                        'nHRhi20Hz', 'nHRhi40Hz', 'nHRhi80Hz',
                        'nFAlo20Hz', 'nFAlo40Hz', 'nFAlo80Hz',
                        'nFAhi20Hz', 'nFAhi40Hz', 'nFAhi80Hz',
                        'meanRTmslo20Hz', 'meanRTmslo40Hz', 'meanRTmslo80Hz',
                        'meanRTmshi20Hz', 'meanRTmshi40Hz', 'meanRTmshi80Hz')
setwd(dir_wd)
write.table(outTable, paste('data_logfiles.tsv',sep = ''), sep="\t", row.names = FALSE)

# Save block order
outBlockorder <- data.frame()
df$load[df$load=="no"] = 1
df$load[df$load=="low"] = 2
df$load[df$load=="high"] = 3
df$load = as.numeric(df$load)
df$tone[df$tone=="20Hz_AM"] = 20
df$tone[df$tone=="40Hz_AM"] = 40
df$tone[df$tone=="80Hz_AM"] = 80
df$tone = as.numeric(df$tone)
df$combined = df$load + df$tone
outBlockorder = aggregate(df$combined, by = list(df$block_nr, df$fp), FUN = "mean")
colnames(outBlockorder) = c('block', 'fp', 'AM_Load')
write.table(outBlockorder, paste('data_blockorder.tsv',sep = ''), sep="\t", row.names = FALSE)

# Process NASA
dir_nasa = "~/Box Sync/ASSR2/NASA"
#dir_nasa = "C:/Users/mszyc/Box Sync/ASSR2/NASA"
#dir_nasa = "~/Malina_Box/Box Sync/ASSR2/NASA"
setwd(dir_nasa)

tmpnasa = data.frame()
for (fp in fps){
  tmp = read.csv(paste0('fp',fp,'_BorgScale.txt'), sep = '\t', header = T, stringsAsFactors = F)
  tmpnasa = rbind(tmpnasa, tmp)
}
rm(fp, tmp)
colnames(tmpnasa)[2] = "block"

D = merge(tmpnasa,outBlockorder, by = c("fp","block"))
D = aggregate(D, by = list(D$fp,D$AM_Load), FUN = "mean")

D = D[with(D, order(fp,AM_Load)), ]
D = D[,-c(1:2,4)]

Dnasa = matrix(NA, nrow = 0, ncol = 19)
for (fp in fps){
  tmp = fp
  for (c in c(21,22,23,41,42,43,81,82,83)){
    tmp = c(tmp, as.numeric(D[D$fp==fp & D$AM_Load==c,2:3]))
  }
  Dnasa = rbind(Dnasa, tmp)
}
Dnasa = as.data.frame(Dnasa, row.names = FALSE)
colnames(Dnasa) = c('fp', 
                  'Mentalno20Hz', 'Effortno20Hz',
                  'Mentallo20Hz', 'Effortlo20Hz', 
                  'Mentalhi20Hz', 'Efforthi20Hz',
                  'Mentalno40Hz', 'Effortno40Hz',
                  'Mentallo40Hz', 'Effortlo40Hz', 
                  'Mentalhi40Hz', 'Efforthi40Hz',
                  'Mentalno80Hz', 'Effortno80Hz',
                  'Mentallo80Hz', 'Effortlo80Hz', 
                  'Mentalhi80Hz', 'Efforthi80Hz')                  
setwd(dir_wd)
write.table(Dnasa, 'data_NASA.tsv', sep="\t", row.names = FALSE)
